# Processing of LuxWater Pharma MS/MS Data with MetFrag
# Emma Schymanski, 1 April 2020

#### Initial set up: Packages, Directories etc ####

library(ReSOLUTION)
library(RChemMass)
library(stringr)

# set up directories etc to start MetFrag processing
# start with positive data in this script (hence _pos in name)
base_dir <- "C:/LuxEau_MetfragData_pos/20200312_LuxWater_pos/"
setwd(base_dir)
msms_dir <- paste0(base_dir,"MS2/")
#Does this exist?
dir.exists(msms_dir)

#This is the location that you saved MetFrag command line version (the jar file)
MetFrag_dir <- "C:/MetFrag/"
dir.exists(MetFrag_dir)


#### MetFrag with PubChemLite - PharmacoBio & Drug Medic Info ######

#Define local database file
localCSV_dir <- "C:/CompoundDBs/"
localDB <- paste0(localCSV_dir,"PubChemLite_tier0.csv")
#test that this is correct...
file.exists(localDB)

#Define where to save the results ...
MetFrag_results_dir <- paste0(base_dir,"MetFrag_PCLite_PBDM_all")
#PBDM= PharmacoBio and DrugMedical Info
#Does this exist?
dir.exists(MetFrag_results_dir)
# First time, this doesn't yet exist, so have to create it
dir.create(MetFrag_results_dir)


# Start of the results file is a summary of calculations, to tell us what to process
# Here, this is the summary file you've made with ShinyScreen (e.g. ftable.csv)
summary_file <- "ftable_state.csv"

# Name the results output file:
summary_results_file <- sub(".csv","All_wMetFragResults_PubChemLite_PBDM.csv",summary_file)
#PBDM= PharmacoBio and DrugMedical Info

# read in the summary file, and select only those with MS/MS
cmpd_info <- read.csv(summary_file, stringsAsFactors = FALSE)
dim(cmpd_info)
# count only those for which we have MS/MS
cmpd_info <- cmpd_info[which(cmpd_info$MS2==TRUE),]
cmpd_info <- cmpd_info[which(!is.na(cmpd_info$MS2rt)),]
# this is still 5406 rows ... ouch.
length(unique(cmpd_info$Name))
suspect_names <- unique(cmpd_info$Name)
# but only 290 names ... cull by name ...
i <- 1
i_names <- vector(length=length(suspect_names))
for (i in 1:length(suspect_names)) {
  i_name <- grep(suspect_names[i],cmpd_info$Name)[1]
  i_names[i] <- i_name
}
# trim to have only one entry per name
#cmpd_info <- cmpd_info[i_names,]
# there are some NAs ..
#which(is.na(cmpd_info$Name))
# remove these NAs ...
#cmpd_info <- cmpd_info[-which(is.na(cmpd_info$Name)),]
#which(is.na(cmpd_info$Name))
# now no NAs remain ...
#dim(cmpd_info)

#Select a run name based on the folder name (or add your own)
run_name <- basename(base_dir)
#run_name <- "my_run_name"


# set up new columns
cmpd_info$msms_avail <- FALSE
cmpd_info$msms_peaks <- ""
#basic MetFrag summary
cmpd_info$num_poss_IDs <- ""
cmpd_info$max_Score <- ""
cmpd_info$n_Score_GE4 <- ""
cmpd_info$n_Score_GE2 <- ""
# summary of top candidate and max metadata values
cmpd_info$SMILES_maxScore <- ""
cmpd_info$Name_maxScore <- ""
cmpd_info$ID_maxScore <- ""
cmpd_info$ExplPeaks_maxScore <- ""
##max of individual categories
#MetFrag
cmpd_info$max_NoExplPeaks <- ""
cmpd_info$NumberPeaksUsed <- ""
cmpd_info$max_FragmenterScore <- ""
#MoNA
cmpd_info$max_MoNAIndiv <- ""
#PCLite terms
cmpd_info$max_PC_Patents <- ""
cmpd_info$max_PC_PubMedRefs <- ""
cmpd_info$max_PC_AnnoTypeCount <- ""
cmpd_info$max_PC_Pharma <- ""
cmpd_info$max_PC_DrugMed <- ""

# #summary of scores over all candidates
cmpd_info$cand_IDs <- ""
cmpd_info$cand_Scores <- ""
cmpd_info$cand_NoExplPeaks <- ""
cmpd_info$cand_FragmenterScore <- ""
#MoNA
cmpd_info$cand_MoNAIndiv <- ""
#PCLite terms
cmpd_info$cand_PC_Patents <- ""
cmpd_info$cand_PC_PubMedRefs <- ""
cmpd_info$cand_PC_AnnoTypeCount <- ""
cmpd_info$cand_PC_Pharma <- ""
cmpd_info$cand_PC_DrugMed <- ""
#Now we've added DrugMedicInfo too ...


#This is all positive data, so set isPos=TRUE here
isPos <- TRUE
# Are the masses neutral or charged?
isMassNeutral <- FALSE

i <- 1 #always test on an individual file first (line for line just skip the "for" bit)
#other interesting is are:
# 57, 115, 131, 133 (132)
#i <- 57

#for (i in c(2:10)) {
for (i in 11:length(cmpd_info$ID)) {
  #retrieve data we need
  cmpdID <- cmpd_info$ID[i]
  ExactMass <- cmpd_info$mz[i] # note this is the CHARGED mass, set above with isMassCharged <- TRUE
  #[this is important for later!]

  # test if we have MSMS
  msms_pattern <- paste0(str_pad(as.character(cmpd_info$ID[i]),6,pad="0"),"_",
                         as.character(cmpd_info$mode[i]), "_",
                         as.character(cmpd_info$tag[i]), "_",
                         as.character(cmpd_info$set[i]), ".csv")
  msms_file_path <- list.files(msms_dir, pattern=msms_pattern,full.names = T)
  msms_file <- msms_file_path

  if (length(msms_file)<1) {
    cmpd_info$msms_peaks[i] <- "N/A"
    msms_file <- paste0(MetFrag_dir,"dummy_msms.txt")
    msms_peaks <- read.table(msms_file)
    # write peaks to MetFrag temp file
    MetFrag_msms <- paste0(MetFrag_dir,"MetFrag_temp_msms.txt")
    write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
  } else {
    cmpd_info$msms_avail[i] <- TRUE
    msms_peaks <- read.csv(msms_file)
    MetFrag_msms <- paste0(MetFrag_dir,"MetFrag_temp_msms.txt")
    write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    msms_peak_line <- ""
    for (j in seq_along(msms_peaks$mz)) {
      msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
    }
    cmpd_info$msms_peaks[i] <- msms_peak_line
  }

  # # now, run MetFrag and extract results for reporting into cmpd_info.
  LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,AnnoTypeCount,PharmacoInfo,DrugMedicInfo"
  LocalCSVScoreWeights <- ",1,1,1,1,1"

  results_filename <- sub(".csv","",msms_pattern)
  #results_filename <- paste0(run_name,"_",as.character(cmpdID))
  if (isPos) {
    config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=isMassNeutral,
                                 results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5,
                                 peaklist_path = MetFrag_msms, base_dir = MetFrag_results_dir,
                                 DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F,
                                 IsPosMode = TRUE,filter_by_InChIKey = T,
                                 UDS_Category = LocalCSVScoreTerms,
                                 UDS_Weights = LocalCSVScoreWeights)
  } else {
    config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=isMassNeutral,
                                 results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5,
                                 peaklist_path = MetFrag_msms, base_dir = MetFrag_results_dir,
                                 DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F,
                                 IsPosMode = FALSE,filter_by_InChIKey = T,
                                 UDS_Category = LocalCSVScoreTerms,
                                 UDS_Weights = LocalCSVScoreWeights)
  }

  runMetFrag(config_file, MetFrag_dir, CL_name = "MetFrag2.4.5-CL.jar")

  results_file <- paste0(MetFrag_results_dir,"/results/",results_filename,".xls")
  #file.exists(results_file)

  #extract results we need:
  MetFrag_res <- suppressWarnings(read_excel(results_file))
  # test if there are results ...
  if (length(MetFrag_res)==0) {
    cmpd_info$num_poss_IDs[i] <- 0
  } else if (length(MetFrag_res)>0) {
    index_maxScore <- 1 #which(max(as.numeric(MetFrag_res$Score))==as.numeric(MetFrag_res$Score))

    #number of candidates and score summaries
    cmpd_info$num_poss_IDs[i] <- length(MetFrag_res$Score)
    cmpd_info$max_Score[i] <- max(as.numeric(MetFrag_res$Score))
    cmpd_info$n_Score_GE4[i] <- length(which(as.numeric(MetFrag_res$Score)>=4))
    cmpd_info$n_Score_GE2[i] <- length(which(as.numeric(MetFrag_res$Score)>=2))

    # summary of top candidate and max metadata values
    cmpd_info$SMILES_maxScore[i] <- MetFrag_res$SMILES[index_maxScore]
    cmpd_info$Name_maxScore[i] <- MetFrag_res$Synonym[index_maxScore]
    cmpd_info$ID_maxScore[i] <- MetFrag_res$Identifier[index_maxScore]
    cmpd_info$ExplPeaks_maxScore[i] <- MetFrag_res$ExplPeaks[index_maxScore]
    ##max of individual categories
    #MetFrag
    cmpd_info$max_NoExplPeaks[i] <- max(as.numeric(MetFrag_res$NoExplPeaks))
    cmpd_info$NumberPeaksUsed[i] <- max(as.numeric(MetFrag_res$NumberPeaksUsed))
    cmpd_info$max_FragmenterScore[i] <- max(as.numeric(MetFrag_res$FragmenterScore))
    #MoNA
    cmpd_info$max_MoNAIndiv[i] <- max(as.numeric(MetFrag_res$OfflineIndividualMoNAScore))
    #PubChemLite
    cmpd_info$max_PC_Patents[i] <- max(as.numeric(MetFrag_res$Patent_Count))
    cmpd_info$max_PC_PubMedRefs[i] <- max(as.numeric(MetFrag_res$PubMed_Count))
    cmpd_info$max_PC_AnnoTypeCount[i] <- max(as.numeric(MetFrag_res$AnnoTypeCount))
    cmpd_info$max_PC_Pharma[i] <- max(as.numeric(MetFrag_res$PharmacoInfo))
    cmpd_info$max_PC_DrugMed[i] <- max(as.numeric(MetFrag_res$DrugMedicInfo))

    #summary of scores over all candidates
    cmpd_info$cand_IDs[i] <- paste(MetFrag_res$Identifier,collapse=";")
    cmpd_info$cand_Scores[i] <- paste(MetFrag_res$Score,collapse=";")
    cmpd_info$cand_NoExplPeaks[i] <- paste(MetFrag_res$NoExplPeaks,collapse=";")
    cmpd_info$cand_FragmenterScore[i] <- paste(MetFrag_res$FragmenterScore,collapse=";")
    #MoNA
    cmpd_info$cand_MoNAIndiv[i] <- paste(MetFrag_res$OfflineIndividualMoNAScore,collapse=";")
    #PCLite terms
    cmpd_info$cand_PC_Patents[i] <- paste(MetFrag_res$Patent_Count,collapse=";")
    cmpd_info$cand_PC_PubMedRefs[i] <- paste(MetFrag_res$PubMed_Count,collapse=";")
    cmpd_info$cand_PC_AnnoTypeCount[i] <- paste(MetFrag_res$AnnoTypeCount,collapse=";")
    cmpd_info$cand_PC_Pharma[i] <- paste(MetFrag_res$PharmacoInfo,collapse=";")
    cmpd_info$cand_PC_DrugMed[i] <- paste(MetFrag_res$DrugMedicInfo,collapse=";")


  }
}

#write.csv(cmpd_info[c(1:10),],summary_results_file,row.names=F)
write.csv(cmpd_info,summary_results_file,row.names=F)


#### END ######
